<?php
// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Grimlock_Tutor
 *
 * @author  themosaurus
 * @since   1.0.0
 * @package grimlock-tutor/inc
 */
class Grimlock_Tutor {
	/**
	 * @var array Default colors to use when tutor colors aren't set
	 */
	private $tutor_default_colors = array(
		'tutor_primary_color'       => 'var(--grimlock-button-primary-background-color)',
		'tutor_primary_hover_color' => 'var(--grimlock-button-primary-hover-background-color)',
		'tutor_text_color'          => 'var(--grimlock-text-color)',
		'tutor_button_primary'      => 'var(--grimlock-button-primary-background-color)',
		'tutor_button_danger'       => 'var(--grimlock-color-danger)',
		'tutor_button_success'      => 'var(--grimlock-color-success)',
		'tutor_button_warning'      => 'var(--grimlock-color-warning)',
	);

	/**
	 * Setup class.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		load_plugin_textdomain( 'grimlock-tutor', false, 'grimlock-tutor/languages' );

		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ), 10 );

		require_once GRIMLOCK_TUTOR_PLUGIN_DIR_PATH . 'inc/customizer/class-grimlock-tutor-archive-courses-customizer.php';
		require_once GRIMLOCK_TUTOR_PLUGIN_DIR_PATH . 'inc/customizer/class-grimlock-tutor-single-courses-customizer.php';

		add_action( 'grimlock_query_courses', array( $this, 'query_courses' ) );

		foreach ( $this->tutor_default_colors as $tutor_option_name => $color_value ) {
			add_filter( $tutor_option_name, array( $this, 'change_tutor_default_color' ) );
		}
	}

	/**
	 * Enqueue scripts
	 */
	public function enqueue_scripts() {
		wp_enqueue_style( 'grimlock-tutor', GRIMLOCK_TUTOR_PLUGIN_DIR_URL . 'assets/css/style.css', array(), GRIMLOCK_TUTOR_VERSION );

		/*
		 * Load style-rtl.css instead of style.css for RTL compatibility
		 */
		wp_style_add_data( 'grimlock-tutor', 'rtl', 'replace' );
	}

	/**
	 * Display the courses component.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args
	 */
	public function query_courses( $args = array() ) {
		$args = apply_filters( 'grimlock_query_courses_args', $args );

		if ( function_exists( 'tutor_load_template' ) ) {
			$change_post_thumbnail_size = function( $size ) use( $args ) {
				return ! empty( $args['post_thumbnail_size'] ) ? $args['post_thumbnail_size'] : $size;
			};

			add_filter( 'tutor_course_thumbnail_size', $change_post_thumbnail_size, 10, 1 );

			tutor_load_template( 'loop.course' );

			remove_filter( 'tutor_course_thumbnail_size', $change_post_thumbnail_size, 10 );
		}
	}

	/**
	 * Change default tutor color option
	 *
	 * @param string $color Color value
	 *
	 * @return string Changed color value
	 */
	public function change_tutor_default_color( $color ) {
		if ( empty( $color ) ) {
			return ! empty( $this->tutor_default_colors[ current_filter() ] ) ? $this->tutor_default_colors[ current_filter() ] : $color;
		}

		return $color;
	}
}