<?php
/**
 * Grimlock_Tutor_Archive_Courses_Customizer Class
 *
 * @author  Themosaurus
 * @since   1.0.0
 * @package grimlock-tutor
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * The Grimlock Customizer class for the courses archive pages.
 */
class Grimlock_Tutor_Archive_Courses_Customizer extends Grimlock_Template_Customizer {

	/**
	 * Setup class.
	 *
	 * @since 1.4.0
	 */
	public function __construct() {
		$this->id         = 'archive_courses';
		$this->section    = 'grimlock_tutor_archive_courses_customizer_section';
		$this->title      = esc_html__( 'Courses Page', 'grimlock-tutor' );

		add_action( 'after_setup_theme',                            array( $this, 'add_customizer_fields'           ), 20    );

		add_filter( 'body_class',                                   array( $this, 'add_body_classes'                ), 10, 1 );

		add_filter( 'grimlock_customizer_controls_js_data',         array( $this, 'add_customizer_controls_js_data' ), 10, 1 );
		add_filter( 'grimlock_content_class',                       array( $this, 'add_content_classes'             ), 10, 1 );
		add_filter( 'grimlock_custom_header_args',                  array( $this, 'add_custom_header_args'          ), 20, 1 );
		add_filter( 'grimlock_custom_header_displayed',             array( $this, 'has_custom_header_displayed'     ), 10, 1 );
		add_filter( 'grimlock_template_sidebar_right_displayed',    array( $this, 'has_sidebar_right_displayed'     ), 20, 1 );
		add_filter( 'grimlock_template_sidebar_left_displayed',     array( $this, 'has_sidebar_left_displayed'      ), 20, 1 );

		add_action( 'get_header',                                   array( $this, 'display_before_content'          ), 10, 2 );
		add_action( 'get_footer',                                   array( $this, 'display_after_content'           ), 10, 2 );

		add_action( 'customize_controls_print_scripts',             array( $this, 'add_scripts'                     ), 30, 1 );

		add_filter( 'grimlock_archive_customizer_is_template',      array( $this, 'archive_customizer_is_template'  ), 10,  1 );
		add_filter( 'grimlock_search_customizer_is_template',       array( $this, 'archive_customizer_is_template'  ), 10,  1 );

		add_filter( 'grimlock_custom_header_customizer_padding_y_field_args',        array( $this, 'add_custom_header_customizer_padding_y_field_description'        ), 10,  1 );
		add_filter( 'grimlock_custom_header_customizer_layout_field_args',           array( $this, 'add_custom_header_customizer_layout_field_description'           ), 10,  1 );
		add_filter( 'grimlock_custom_header_customizer_container_layout_field_args', array( $this, 'add_custom_header_customizer_container_layout_field_description' ), 10,  1 );

		add_filter( 'grimlock_tutor_custom_header_background_image', array( $this, 'custom_header_background_image' ) );
	}

	/**
	 * Add tabs to the Customizer to group controls.
	 *
	 * @since 1.0.0
	 *
	 * @param  array $js_data The array of data for the Customizer controls.
	 *
	 * @return array          The filtred array of data for the Customizer controls.
	 */
	public function add_customizer_controls_js_data( $js_data ) {
		$js_data['tabs'][$this->section] = array(
			array(
				'label' => esc_html__( 'General', 'grimlock-tutor' ),
				'class' => 'archive_courses-general-tab',
				'controls' => array(
					'archive_courses_title',
					'archive_courses_description',
					"{$this->section}_divider_30",
					"{$this->section}_heading_30",
					'archive_courses_custom_header_displayed',
				),
			),
			array(
				'label' => esc_html__( 'Layout', 'grimlock-tutor' ),
				'class' => 'archive_courses-layout-tab',
				'controls' => array(
					'archive_courses_custom_header_layout',
					"{$this->section}_divider_110",
					'archive_courses_custom_header_container_layout',
					"{$this->section}_divider_120",
					'archive_courses_layout',
					"{$this->section}_divider_140",
					'archive_courses_sidebar_mobile_displayed',
					'archive_courses_container_layout',
				),
			),
			array(
				'label' => esc_html__( 'Style', 'grimlock-tutor' ),
				'class' => 'archive_courses-style-tab',
				'controls' => array(
					'archive_courses_custom_header_background_image',
					"{$this->section}_divider_210",
					'archive_courses_custom_header_padding_y',
					"{$this->section}_divider_220",
					'archive_courses_content_padding_y',
				),
			),
		);
		return $js_data;
	}

	/**
	 * Register default values, settings and custom controls for the Theme Customizer.
	 *
	 * @since 1.4.0
	 */
	public function add_customizer_fields() {
		$post_type_obj       = get_post_type_object( 'courses' );
		$archive_title       = esc_html__( 'Courses', 'grimlock-tutor' );
		$archive_description = '';

		if ( is_object( $post_type_obj ) && isset( $post_type_obj->label ) && $post_type_obj->label !== '' ) {
			$archive_title = $post_type_obj->label;
		}

		if ( function_exists( 'get_tutor_option' ) ) {
			$course_archive_page = get_tutor_option( 'course_archive_page' );
			if ( $course_archive_page && $course_archive_page !== '-1' ) {
				$archive_title = get_the_title( $course_archive_page );
				$archive_description = get_post( $course_archive_page ) ? get_post( $course_archive_page )->post_excerpt : '';
			}
		}

		$this->defaults = apply_filters( 'grimlock_tutor_archive_courses_customizer_defaults', array(
			'archive_courses_title'                             => $archive_title,
			'archive_courses_description'                       => $archive_description,
			'archive_courses_custom_header_displayed'           => has_header_image(),

			'archive_courses_custom_header_layout'              => '6-6-cols-left-reverse',
			'archive_courses_custom_header_container_layout'    => 'classic',
			'archive_courses_layout'                            => '12-cols-left',
			'archive_courses_sidebar_mobile_displayed'          => true,
			'archive_courses_container_layout'                  => 'classic',

			'archive_courses_custom_header_background_image'    => get_header_image(),
			'archive_courses_custom_header_padding_y'           => GRIMLOCK_SECTION_PADDING_Y,
			'archive_courses_content_padding_y'                 => GRIMLOCK_CONTENT_PADDING_Y,
		) );

		$this->add_section();

		$this->add_title_field(                          array( 'priority' => 10  ) );
		$this->add_description_field(                    array( 'priority' => 20  ) );
		$this->add_divider_field(                        array( 'priority' => 30  ) );
		$this->add_heading_field(                        array( 'priority' => 30, 'label' => esc_html__( 'Display', 'grimlock-tutor' ) ) );
		$this->add_custom_header_displayed_field(        array( 'priority' => 40  ) );

		$this->add_custom_header_layout_field(           array( 'priority' => 100 ) );
		$this->add_divider_field(                        array( 'priority' => 110 ) );
		$this->add_custom_header_container_layout_field( array( 'priority' => 110 ) );
		$this->add_divider_field(                        array( 'priority' => 120 ) );
		$this->add_layout_field(                         array( 'priority' => 120 ) );
		$this->add_sidebar_mobile_displayed_field(       array( 'priority' => 130 ) );
		$this->add_divider_field(                        array( 'priority' => 140 ) );
		$this->add_container_layout_field(               array( 'priority' => 140 ) );

		$this->add_custom_header_background_image_field( array( 'priority' => 200 ) );
		$this->add_divider_field(                        array( 'priority' => 210 ) );
		$this->add_custom_header_padding_y_field(        array( 'priority' => 210 ) );
		$this->add_divider_field(                        array( 'priority' => 220 ) );
		$this->add_content_padding_y_field(              array( 'priority' => 220 ) );
	}

	/**
	 * Add a Kirki section in the Customizer.
	 *
	 * @since 1.4.0
	 *
	 * @param array $args
	 */
	protected function add_section( $args = array() ) {
		if ( class_exists( 'Kirki' ) ) {
			Kirki::add_panel( 'grimlock_tutor_customizer_panel', array(
				'priority' => 120,
				'title'    => esc_html__( 'Tutor LMS', 'grimlock-tutor' ),
			) );

			Kirki::add_section( $this->section, apply_filters( "{$this->section}_args", array(
				'title'    => $this->title,
				'priority' => isset( $args['priority'] ) ? $args['priority'] :  10,
				'panel'    => 'grimlock_tutor_customizer_panel',
			) ) );
		}
	}

	/**
	 * Add a Kirki text field to set the title in the Customizer.
	 *
	 * @since 1.4.0
	 *
	 * @param array $args
	 */
	protected function add_title_field( $args = array() ) {
		if ( class_exists( 'Kirki' ) ) {
			$args = wp_parse_args( $args, array(
				'type'              => 'text_disabled',
				'label'             => esc_html__( 'Title', 'grimlock-tutor' ),
				'description'       => esc_html__( 'You can change the header title by choosing a "Course Archive Page" in Tutor LMS settings.', 'grimlock-tutor' ),
				'section'           => $this->section,
				'settings'          => 'archive_courses_title',
				'default'           => $this->get_default( 'archive_courses_title' ),
				'priority'          => 10,
				'sanitize_callback' => 'wp_kses_post',
			) );

			Kirki::add_field( 'grimlock', apply_filters( 'grimlock_tutor_archive_courses_customizer_title_field_args', $args ) );
		}
	}

	/**
	 * Add a Kirki textarea field to set the description in the Customizer.
	 *
	 * @since 1.4.0
	 *
	 * @param array $args
	 */
	protected function add_description_field( $args = array() ) {
		if ( class_exists( 'Kirki' ) ) {
			$args = wp_parse_args( $args, array(
				'type'              => 'textarea_disabled',
				'label'             => esc_html__( 'Description', 'grimlock-tutor' ),
				'description'       => esc_html__( 'You can change the header description by choosing a "Course Archive Page" in Tutor LMS settings, then changing the excerpt of that page.', 'grimlock-tutor' ),
				'section'           => $this->section,
				'settings'          => 'archive_courses_description',
				'default'           => $this->get_default( 'archive_courses_description' ),
				'priority'          => 10,
				'sanitize_callback' => 'wp_kses_post',
			) );

			Kirki::add_field( 'grimlock', apply_filters( 'grimlock_tutor_archive_courses_customizer_description_field_args', $args ) );
		}
	}

	/**
	 * Add a Kirki image field to set the background image for the Custom Header in the Customizer.
	 *
	 * @since 1.4.0
	 *
	 * @param array $args The array of arguments for the Kirki field.
	 */
	protected function add_custom_header_background_image_field( $args = array() ) {
		if ( class_exists( 'Kirki' ) ) {
			$args = wp_parse_args( $args, array(
				'type'     => 'image',
				'section'  => $this->section,
				'label'    => esc_html__( 'Header image', 'grimlock-tutor' ),
				'settings' => 'archive_courses_custom_header_background_image',
				'default'  => $this->get_default( 'archive_courses_custom_header_background_image' ),
				'priority' => 10,
			) );

			Kirki::add_field( 'grimlock', apply_filters( 'grimlock_tutor_archive_courses_customizer_background_image_field_args', $args ) );
		}
	}

	/**
	 * Add arguments using theme mods to customize the Custom Header.
	 *
	 * @since 1.4.0
	 *
	 * @param array $args The default arguments to render the Custom Header.
	 *
	 * @return array      The arguments to render the Custom Header.
	 */
	public function add_custom_header_args( $args ) {
		$args = parent::add_custom_header_args( $args );

		if ( $this->is_template() ) {
			$args['title']    = get_the_archive_title();
			$args['subtitle'] = '';

			if ( function_exists( 'get_tutor_option' ) ) {
				$course_archive_page = get_tutor_option( 'course_archive_page' );
				if ( $course_archive_page && $course_archive_page !== '-1' ) {
					$args['title']    = get_the_title( $course_archive_page );
					$args['subtitle'] = get_post( $course_archive_page ) ? get_post( $course_archive_page )->post_excerpt : '';
				}
			}

			$args['background_image'] = $this->get_theme_mod( 'archive_courses_custom_header_background_image' );
		}

		return $args;
	}

	/**
	 * Check whether Custom Header has to be displayed.
	 *
	 * @since 1.4.0
	 *
	 * @return bool True when Custom Header has to be displayed, false otherwise.
	 */
	public function has_custom_header_displayed( $default ) {
		if ( $this->is_template() ) {
			return true == $this->get_theme_mod( 'archive_courses_custom_header_displayed' );
		}
		return $default;
	}

	/**
	 * Display left sidebar before content
	 *
	 * @param string $name Name for the header
	 * @param array $args Args for the header
	 */
	public function display_before_content( $name, $args ) {
		if ( $this->is_template() ) {
			// Prevent infinite loop
			remove_action( 'get_header', array( $this, 'display_before_content' ), 10 );

			get_header( $name, $args );
			get_sidebar( 'left' );
			?>
			<div id="primary" class="content-area region__col region__col--2">
				<main id="main" class="site-main">
			<?php
		}
	}

	/**
	 * Display right sidebar after content
	 *
	 * @param string $name Name for the footer
	 * @param array $args Args for the footer
	 */
	public function display_after_content( $name, $args ) {
		if ( $this->is_template() ) {
			// Prevent infinite loop
			remove_action( 'get_footer', array( $this, 'display_after_content' ), 10 );

			?>
				</main><!-- #main -->
			</div><!-- #primary -->
			<?php
			get_sidebar( 'right' );
			get_footer( $name, $args );
		}
	}

	/**
	 * Check if the current template is the expected template, the docs page or a similar template.
	 *
	 * @since 1.4.0
	 *
	 * @return bool True when the template is the expected template, false otherwise.
	 */
	protected function is_template() {
		$is_template = is_post_type_archive( 'courses' );
		return apply_filters( 'grimlock_tutor_archive_courses_customizer_is_template', $is_template );
	}

	/**
	 * Disinherit archive customizer settings
	 *
	 * @param bool $default True if we are on a default archive page
	 *
	 * @return bool
	 */
	public function archive_customizer_is_template( $default ) {
		return $default && ! $this->is_template();
	}

	/**
	 * Return the custom header background image to use on docs templates
	 *
	 * @param string $background_image Background image url
	 *
	 * @return string Modified background image url
	 */
	public function custom_header_background_image( $background_image ) {
		return $this->get_theme_mod( 'archive_courses_custom_header_background_image' );
	}

	/**
	 * Add a link in the description pointing to the job listing archive vertical padding field.
	 *
	 * @since 1.4.0
	 *
	 * @param  array $args The array of arguments for the Kirki field.
	 *
	 * @return array       The updated array of arguments for the Kirki field.
	 */
	public function add_custom_header_customizer_padding_y_field_description( $args ) {
		$args['description'] .= wp_kses( __( ', <a href="#archive_courses_custom_header_padding_y" rel="tc-control">Courses Page</a>', 'grimlock-tutor' ), array(
			'a' => array(
				'href' => array(),
				'rel'  => array(),
			),
		) );
		return $args;
	}

	/**
	 * Add a link in the description pointing to the job listing archive layout field.
	 *
	 * @since 1.4.0
	 *
	 * @param  array $args The array of arguments for the Kirki field.
	 *
	 * @return array       The updated array of arguments for the Kirki field.
	 */
	public function add_custom_header_customizer_layout_field_description( $args ) {
		$args['description'] .= wp_kses( __( ', <a href="#archive_courses_custom_header_layout" rel="tc-control">Courses Page</a>', 'grimlock-tutor' ), array(
			'a' => array(
				'href' => array(),
				'rel'  => array(),
			),
		) );
		return $args;
	}

	/**
	 * Add a link in the description pointing to the job listing archive container layout field.
	 *
	 * @since 1.4.0
	 *
	 * @param  array $args The array of arguments for the Kirki field.
	 *
	 * @return array       The updated array of arguments for the Kirki field.
	 */
	public function add_custom_header_customizer_container_layout_field_description( $args ) {
		$args['description'] .= wp_kses( __( ', <a href="#archive_courses_custom_header_container_layout" rel="tc-control">Courses Page</a>', 'grimlock-tutor' ), array(
			'a' => array(
				'href' => array(),
				'rel'  => array(),
			),
		) );
		return $args;
	}

	/**
	 * Add scripts to improve user experience in the customizer
	 */
	public function add_scripts() {
		$archive_url = get_post_type_archive_link( 'courses' );
		if ( function_exists( 'get_tutor_option' ) ) {
			$course_archive_page = get_tutor_option( 'course_archive_page' );
			if ( $course_archive_page && $course_archive_page !== '-1' ) {
				$archive_url = get_permalink( $course_archive_page );
			}
		}
		?>
		<script type="text/javascript">
            jQuery( document ).ready( function( $ ) {
                wp.customize.section( '<?php echo esc_js( $this->section ); ?>', function( section ) {
                    section.expanded.bind( function( isExpanded ) {
                        var previewUrl = '<?php echo esc_js( trailingslashit( $archive_url ) ); ?>';
                        if ( isExpanded && wp.customize.previewer.previewUrl.get() !== previewUrl ) {
                            wp.customize.previewer.previewUrl.set( previewUrl );
                        }
                    } );
                } );
            } );
		</script>
		<?php
	}
}

return new Grimlock_Tutor_Archive_Courses_Customizer();
