<?php
/**
 * Grimlock_Tutor_Single_Courses_Customizer Class
 *
 * @author  Themosaurus
 * @since   1.0.0
 * @package grimlock-tutor
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * The Grimlock Customizer class for the courses single pages.
 */
class Grimlock_Tutor_Single_Courses_Customizer extends Grimlock_Singular_Template_Customizer {

	/**
	 * Setup class.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		$this->id      = 'single_courses';
		$this->section = 'grimlock_tutor_single_courses_customizer_section';
		$this->title   = esc_html__( 'Single Course', 'grimlock-tutor' );

		add_action( 'after_setup_theme',                         array( $this, 'add_customizer_fields'           ), 20    );

		add_filter( 'body_class',                                array( $this, 'add_body_classes'                ), 10, 1 );

		add_filter( 'grimlock_customizer_controls_js_data',      array( $this, 'add_customizer_controls_js_data' ), 10, 1 );
		add_filter( 'grimlock_content_class',                    array( $this, 'add_content_classes'             ), 10, 1 );
		add_filter( 'grimlock_custom_header_args',               array( $this, 'add_custom_header_args'          ), 20, 1 );
		add_filter( 'grimlock_custom_header_displayed',          array( $this, 'has_custom_header_displayed'     ), 10, 1 );
		add_filter( 'grimlock_template_sidebar_right_displayed', array( $this, 'has_sidebar_right_displayed'     ), 10, 1 );
		add_filter( 'grimlock_template_sidebar_left_displayed',  array( $this, 'has_sidebar_left_displayed'      ), 10, 1 );
		add_filter( 'grimlock_single_customizer_is_template',    array( $this, 'single_customizer_is_template'   ), 10, 1 );

		add_action( 'tutor_course/single/before/wrap',           array( $this, 'display_before_content'          ), 10    );
		add_action( 'tutor_course/single/after/wrap',            array( $this, 'display_after_content'           ), 10    );
	}

	/**
	 * Add custom classes to body to modify layout.
	 *
	 * @param $classes
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function add_body_classes( $classes ) {
		$classes = parent::add_body_classes( $classes );

		if ( $this->is_template() ) {
			$displayed_theme_mods = array(
				'single_courses_title_displayed',
			);

			foreach ( $displayed_theme_mods as $theme_mod ) {
				if ( ! empty( $this->get_theme_mod( $theme_mod ) ) ) {
					$classes[] = "grimlock-tutor--{$theme_mod}";
				}
			}
		}

		return $classes;
	}

	/**
	 * Add tabs to the Customizer to group controls.
	 *
	 * @param  array $js_data The array of data for the Customizer controls.
	 *
	 * @return array          The filtered array of data for the Customizer controls.
	 */
	public function add_customizer_controls_js_data( $js_data ) {
		$js_data['tabs'][$this->section] = array(
			array(
				'label' => esc_html__( 'General', 'grimlock-tutor' ),
				'class' => 'single_courses-general-tab',
				'controls' => array(
					"{$this->section}_heading_10",
					'single_courses_custom_header_displayed',
					"{$this->section}_divider_30",
					"{$this->section}_heading_30",
					'single_courses_title_displayed',
				),
			),
			array(
				'label' => esc_html__( 'Layout', 'grimlock-tutor' ),
				'class' => 'single_courses-layout-tab',
				'controls' => array(
					'single_courses_custom_header_layout',
					"{$this->section}_divider_110",
					'single_courses_custom_header_container_layout',
					"{$this->section}_divider_120",
					'single_courses_layout',
					'single_courses_sidebar_mobile_displayed',
					"{$this->section}_divider_140",
					'single_courses_container_layout',
				),
			),
			array(
				'label' => esc_html__( 'Style', 'grimlock-tutor' ),
				'class' => 'single_courses-style-tab',
				'controls' => array(
					'single_courses_custom_header_padding_y',
					"{$this->section}_divider_210",
					'single_courses_content_padding_y',
				),
			),
		);
		return $js_data;
	}

	/**
	 * Register default values, settings and custom controls for the Theme Customizer.
	 *
	 * @since 1.0.0
	 */
	public function add_customizer_fields() {
		$this->defaults = apply_filters( 'grimlock_tutor_single_courses_customizer_defaults', array(
			'single_courses_custom_header_displayed'            => false,
			'single_courses_title_displayed'                    => true,

			'single_courses_custom_header_layout'               => '12-cols-center',
			'single_courses_custom_header_container_layout'     => 'classic',
			'single_courses_layout'                             => '12-cols-left',
			'single_courses_sidebar_mobile_displayed'           => true,
			'single_courses_container_layout'                   => 'classic',

			'single_courses_custom_header_padding_y'            => GRIMLOCK_SECTION_PADDING_Y,
			'single_courses_content_padding_y'                  => GRIMLOCK_CONTENT_PADDING_Y,
		) );

		$this->add_section();

		$this->add_heading_field(                            array( 'priority' => 10, 'label' => esc_html__( 'Header Display', 'grimlock' ) ) );
		$this->add_custom_header_displayed_field(            array( 'priority' => 20 ) );
		$this->add_divider_field(                            array( 'priority' => 30 ) );
		$this->add_heading_field(                            array( 'priority' => 30, 'label' => esc_html__( 'Content Display', 'grimlock' ) ) );
		$this->add_title_displayed_field(                    array( 'priority' => 40  ) );

		$this->add_custom_header_layout_field(               array( 'priority' => 100 ) );
		$this->add_divider_field(                            array( 'priority' => 110 ) );
		$this->add_custom_header_container_layout_field(     array( 'priority' => 110 ) );
		$this->add_divider_field(                            array( 'priority' => 120 ) );
		$this->add_layout_field(                             array( 'priority' => 120 ) );
		$this->add_sidebar_mobile_displayed_field(           array( 'priority' => 130 ) );
		$this->add_divider_field(                            array( 'priority' => 140 ) );
		$this->add_container_layout_field(                   array( 'priority' => 140 ) );

		$this->add_custom_header_padding_y_field(            array( 'priority' => 200 ) );
		$this->add_divider_field(                            array( 'priority' => 210 ) );
		$this->add_content_padding_y_field(                  array( 'priority' => 210 ) );
	}

	/**
	 * Add a Kirki checkbox to set the title display in the Customizer.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args
	 */
	protected function add_title_displayed_field( $args = array() ) {
		if ( class_exists( 'Kirki' ) ) {
			$args = wp_parse_args( $args, array(
				'type'     => 'checkbox',
				'section'  => $this->section,
				'label'    => esc_html__( 'Display title', 'grimlock-tutor' ),
				'settings' => "{$this->id}_title_displayed",
				'default'  => $this->get_default( "{$this->id}_title_displayed" ),
				'priority' => 10,
			) );

			Kirki::add_field( 'grimlock', apply_filters( "grimlock_{$this->id}_customizer_title_displayed_field_args", $args ) );
		}
	}

	/**
	 * Add a Kirki section in the Customizer.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args
	 */
	protected function add_section( $args = array() ) {
		if ( class_exists( 'Kirki' ) ) {
			Kirki::add_section( $this->section, apply_filters( "{$this->section}_args", array(
				'title'    => $this->title,
				'priority' => 30,
				'panel'    => 'grimlock_tutor_customizer_panel',
			) ) );
		}
	}

	/**
	 * Add arguments using theme mods to customize the Custom Header.
	 *
	 * @param array $args The default arguments to render the Custom Header.
	 *
	 * @return array      The arguments to render the Custom Header.
	 */
	public function add_custom_header_args( $args ) {
		$args = parent::add_custom_header_args( $args );

		if ( $this->is_template() ) {
			$args['background_image'] = apply_filters( 'grimlock_tutor_custom_header_background_image', '' );
		}

		return $args;
	}

	/**
	 * Check if the current template is the expected template.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True when the template is the expected template, false otherwise.
	 */
	protected function is_template() {
		$is_template = is_singular( array( 'courses', 'lesson', 'tutor_quiz' ) );
		return apply_filters( 'grimlock_tutor_single_courses_customizer_is_template', $is_template );
	}

	/**
	 * Disinherit single customizer settings
	 *
	 * @param bool $default True if we are on a default single page
	 *
	 * @return bool
	 */
	public function single_customizer_is_template( $default ) {
		return $default && ! $this->is_template();
	}

	/**
	 * Display left sidebar before content
	 */
	public function display_before_content() {
		get_sidebar( 'left' );
		?>
		<div id="primary" class="content-area region__col region__col--2">
			<main id="main" class="site-main">
		<?php
	}

	/**
	 * Display right sidebar after content
	 */
	public function display_after_content() {
		?>
			</main><!-- #main -->
		</div><!-- #primary -->
		<?php
		get_sidebar( 'right' );
	}
}

return new Grimlock_Tutor_Single_Courses_Customizer();
